@@ -3,35 +3,36 @@
 
 from __future__ import annotations
 
-__all__: list[str] = []
-
-from .imagecodecs import (
-    __doc__,
-    __version__,
-    __getattr__,
+from .imagecodecs import (  # jpeg_check,; jpeg_version,; JpegError,; JPEG,
+    NONE,
+    NUMPY,
+    DelayedImportError,
+    NoneError,
+    NumpyError,
+    __all__,
     __dir__,
+    __doc__,
+    __getattr__,
+    __version__,
+    _codecs,
     _extensions,
-    version,
+    imagefileext,
     imread,
     imwrite,
-    imagefileext,
-    DelayedImportError,
-    none_encode,
-    none_decode,
-    none_check,
-    none_version,
-    NoneError,
-    NONE,
-    numpy_encode,
-    numpy_decode,
-    numpy_check,
-    numpy_version,
-    NumpyError,
-    NUMPY,
-    jpeg_encode,
     jpeg_decode,
-    # jpeg_check,
-    # jpeg_version,
-    # JpegError,
-    # JPEG,
+    jpeg_encode,
+    none_check,
+    none_decode,
+    none_encode,
+    none_version,
+    numpy_check,
+    numpy_decode,
+    numpy_encode,
+    numpy_version,
+    version,
 )
+
+# constants are repeated for documentation
+
+__version__ = __version__
+"""Imagecodecs version string."""
